/**
 ****************************************************************************************
 *
 * @file app_led_switch.c
 *
 * @brief LED Switch Application Module Entry point
 *
 * Copyright (C) RivieraWaves 2009-2015
 *
 *
 ****************************************************************************************
 */

/**
 ****************************************************************************************
 * @addtogroup APP
 * @{
 ****************************************************************************************
 */

#include "rwip_config.h" // SW configuration

/*
 * INCLUDE FILES
 ****************************************************************************************
 */

#include "app.h"             // Application Manager Definitions
#include "app_led_switch.h"  // LED Switch Service Application Definitions
#include "led_switch_task.h" // LED Switch Profile Functions
#include "prf_types.h"       // Profile Common Types Definitions
#include "uart.h"
#include "uart2.h"

extern bool uart_tp_mode_flag;
/*
 * LOCAL FUNCTION DEFINITIONS
 ****************************************************************************************
 */
static int led_switch_value_req_ind_handler(ke_msg_id_t const msgid,
                                            struct led_switch_value_req_ind const *param,
                                            ke_task_id_t const dest_id,
                                            ke_task_id_t const src_id)
{
    const uint8_t led_on = 1, led_off = 0;
    // Initialize length
    uint8_t len = 0;
    // Pointer to the data
    uint8_t *data = NULL;

    at_rsp_content *rsp_content;

    // Check requested value
    if (uart_tp_mode_flag == false)
    {
        switch (param->value)
        {
        case LED_ON_OFF_STATUS:
        {
            // AT command
            UART_SEND_DATA(AT_CMD_IO_GET);
            // Wait for response
            rsp_content = at_wait_for_rsp();
            // Set information
            len = APP_LED_SWITCH_LEN;
            if (rsp_content->data[4] == 0x31)
            {
                data = (uint8_t *)&led_off;
            }
            else
            {
                data = (uint8_t *)&led_on;
            }
        }
        break;

        default:
            ASSERT_ERR(0);
            break;
        }
    }

    // Allocate confirmation to send the value
    struct led_switch_value_cfm *cfm_value = KE_MSG_ALLOC_DYN(LED_SWITCH_VALUE_CFM,
                                                              src_id, dest_id,
                                                              led_switch_value_cfm,
                                                              len);

    // Set parameters
    cfm_value->value = param->value;
    cfm_value->length = len;
    if (len)
    {
        // Copy data
        memcpy(&cfm_value->data[0], data, len);
    }
    // Send message
    ke_msg_send(cfm_value);

    return (KE_MSG_CONSUMED);
}

/*
 * GLOBAL FUNCTION DEFINITIONS
 ****************************************************************************************
 */

void app_led_switch_init(void)
{
    // Nothing to do
}

void app_led_switch_add_switches(void)
{
    struct led_switch_db_cfg *db_cfg;
    // Allocate the CUSTOM_CREATE_DB_REQ
    struct gapm_profile_task_add_cmd *req = KE_MSG_ALLOC_DYN(GAPM_PROFILE_TASK_ADD_CMD,
                                                             TASK_GAPM, TASK_APP,
                                                             gapm_profile_task_add_cmd, sizeof(struct led_switch_db_cfg));
    // Fill message
    req->operation = GAPM_PROFILE_TASK_ADD;
    req->sec_lvl = 0;
    req->prf_task_id = TASK_ID_LED_SWITCH;
    req->app_task = TASK_APP;
    req->start_hdl = 0;

    // Set parameters
    db_cfg = (struct led_switch_db_cfg *)req->param;
    db_cfg->features = APP_LED_SWITCH_FEATURES;

    // Send the message
    ke_msg_send(req);
}

/*
 * GLOBAL VARIABLE DEFINITIONS
 ****************************************************************************************
 */

/// Default State handlers definition
const struct ke_msg_handler app_led_switch_msg_handler_list[] =
    {
        {LED_SWITCH_VALUE_REQ_IND, (ke_msg_func_t)led_switch_value_req_ind_handler},
};

const struct ke_state_handler app_led_switch_table_handler =
    {&app_led_switch_msg_handler_list[0], (sizeof(app_led_switch_msg_handler_list) / sizeof(struct ke_msg_handler))};

/// @} APP
