/**
 ****************************************************************************************
 *
 * @file uart_tp.h
 *
 * @brief Header file - UART transparent Service Server Role
 *
 * Copyright (C) beken 2009-2015
 *
 *
 ****************************************************************************************
 */
#ifndef _UART_TP_H_
#define _UART_TP_H_

/**
 ****************************************************************************************
 * @addtogroup  UART transparent 'Profile' Server
 * @ingroup UART transparent
 * @brief UART transparent 'Profile' Server
 * @{
 ****************************************************************************************
 */

/*
 * INCLUDE FILES
 ****************************************************************************************
 */

#include "rwip_config.h"
#include "rwprf_config.h"

#if (BLE_UART_TP_SERVER)

#include "tp_mode_task.h"
#include "atts.h"
#include "prf_types.h"
#include "prf.h"


/*
 * DEFINES
 ****************************************************************************************
 */

#define UART_TP_CFG_FLAG_MANDATORY_MASK       (0x3F)
#define UART_TP_CFG_FLAG_NTF_SUP_MASK         (0x08)

#define UART_TP_SERVICE_UUID                  (0xF000)
#define UART_TX_CHAR_UUID                     (0xF001)
#define UART_RX_CHAR_UUID                     (0xF002)

#define TP_ATT_BT_UUID_128( uuid )            {0x5d, 0x32, 0x86, 0x40, 0x3b, 0x10, 0x11, 0xee, \
                                               0xbe, 0x56, 0x02, 0x42, 0xac, 0x12, LO_UINT16( uuid ), HI_UINT16( uuid )}
enum
{		
		ATT_UART_TP_SERVER_SERVICE 						= ATT_UUID_16(UART_TP_SERVICE_UUID),	  
		ATT_UART_TP_SERVER_CHAR_TX            = ATT_UUID_16(UART_TX_CHAR_UUID),		
		ATT_UART_TP_SERVER_CHAR_RX					  = ATT_UUID_16(UART_RX_CHAR_UUID),		
};

/// Battery Service Attributes Indexes
enum
{
	UART_TP_IDX_SVC,

	UART_TP_RX_CHAR,
	UART_TP_RX_PAYLOAD,

	UART_TP_TX_CHAR,
	UART_TP_TX_PAYLOAD,
	UART_TP_TX_NTF_CFG,

	UART_TP_IDX_NB,
};

/*
 * TYPE DEFINITIONS
 ****************************************************************************************
 */


/// UART transparent 'Profile' Server environment variable
struct uart_tp_env_tag
{
    /// profile environment
    prf_env_t prf_env;
   
    /// On-going operation
    struct ke_msg * operation;
    /// UART transparent Services Start Handle
    uint16_t start_hdl;
    /// Level of the FFF1
    uint8_t uart_tx_payload[UART_TX_PAYLOAD_LEN];
	
    uint8_t uart_rx_payload[UART_RX_PAYLOAD_LEN];
    /// BASS task state
    ke_state_t state[UART_TP_IDX_MAX];
    /// Notification configuration of peer devices.
    uint8_t ntf_cfg[BLE_CONNECTION_MAX];
    /// Database features
    uint8_t features;

};



/**
 ****************************************************************************************
 * @brief Retrieve tp service profile interface
 *
 * @return tp service profile interface
 ****************************************************************************************
 */
const struct prf_task_cbs* uart_tp_prf_itf_get(void);
uint16_t uart_tp_get_att_handle(uint8_t att_idx);
uint8_t  uart_tp_get_att_idx(uint16_t handle, uint8_t *att_idx);
void uart_notify_tx_payload(struct uart_tp_env_tag* uart_tp_env, struct uart_tx_payload_upd_req const *param);
#endif /* #if (BLE_UART_TP_SERVER) */
#endif /*  _UART_TP_H_ */
