/**
  **************************************************************************
  * @file     tp_mode.c
  * @version  v2.0.3
  * @date     2022-08-16
  * @brief    tp_mode program
  **************************************************************************
  *                       Copyright notice & Disclaimer
  *
  * The software Board Support Package (BSP) that is made available to
  * download from Artery official website is the copyrighted work of Artery.
  * Artery authorizes customers to use, copy, and distribute the BSP
  * software and its related documentation for the purpose of design and
  * development in conjunction with Artery microcontrollers. Use of the
  * software is governed by this copyright notice and the following disclaimer.
  *
  * THIS SOFTWARE IS PROVIDED ON "AS IS" BASIS WITHOUT WARRANTIES,
  * GUARANTEES OR REPRESENTATIONS OF ANY KIND. ARTERY EXPRESSLY DISCLAIMS,
  * TO THE FULLEST EXTENT PERMITTED BY LAW, ALL EXPRESS, IMPLIED OR
  * STATUTORY OR OTHER WARRANTIES, GUARANTEES OR REPRESENTATIONS,
  * INCLUDING BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY,
  * FITNESS FOR A PARTICULAR PURPOSE, OR NON-INFRINGEMENT.
  *
  **************************************************************************
  */
  
#include "stdio.h"
#include "string.h"
#include "tp_mode.h"
#include "at32wb415_board.h"
#include "usb_core.h"
#include "custom_hid_class.h"
#include "custom_hid_desc.h"

extern otg_core_type otg_core_struct;
extern uint8_t report_buf[USBD_CUSTOM_IN_MAXPACKET_SIZE];
extern flag_status USB_TP_MODE;
extern uint32_t uart_tx_total_length;

uartstruct tp_mode_rx_uart = {0};
uartstruct tp_mode_tx_uart = {0};

/**
  * @brief  this function handle output data that received by usart3
  * @param  none
  * @retval none
  */
void tp_mode_rx_handler(void)
{
  uint16_t i;
  
  if(tp_mode_rx_uart.count > 0)
  {
    usart_interrupt_enable(USART3, USART_RDBF_INT, FALSE);
    tp_mode_rx_uart.count--;    
    usart_interrupt_enable(USART3, USART_RDBF_INT, TRUE);
    tp_mode_rx_uart.tail++;
    if(tp_mode_rx_uart.buf[tp_mode_rx_uart.tail-2] == 0x0d  && tp_mode_rx_uart.buf[tp_mode_rx_uart.tail-1] == 0x0a)
    {
      tp_mode_rx_uart.recv_end = SET;
      /* output as usb report, CR/LF are not required */
      if(USB_TP_MODE == SET)
      {
        report_buf[0] = HID_REPORT_ID_6;
        report_buf[1] = tp_mode_rx_uart.tail - 2;
        for(i = 0; i < report_buf[1]; i++)
        {
          report_buf[i+2] = (uint8_t)tp_mode_rx_uart.buf[i];
        }
        custom_hid_class_send_report(&otg_core_struct.dev, report_buf, USBD_CUSTOM_IN_MAXPACKET_SIZE);
      }
      /* output as usart2 transmit, CR/LF are not required */
      else
      {
        for(i = 0; i < tp_mode_rx_uart.tail - 2; i++)
        {
          while(usart_flag_get(USART2, USART_TDBE_FLAG) != SET);
          usart_data_transmit(USART2, tp_mode_rx_uart.buf[i]);
        }
      }
      memset(&tp_mode_rx_uart, 0, sizeof(tp_mode_rx_uart));
    }

    if(tp_mode_rx_uart.tail > (USART_RECV_LEN - 1))
    {
      tp_mode_rx_uart.tail = 0;
    }
  }
}

/**
  * @brief  this function handle input data that received by usart2
  * @param  none
  * @retval none
  */
void tp_mode_tx_handler(void)
{
  uint16_t i;
  if(tp_mode_tx_uart.count > 0)
  {    
    if(tp_mode_tx_uart.count >2)
    {
      if(tp_mode_tx_uart.buf[tp_mode_tx_uart.count-2] == 0x0d  && tp_mode_tx_uart.buf[tp_mode_tx_uart.count-1] == 0x0a)
      {
        tp_mode_tx_uart.recv_end = SET;
      }
    }

    if(tp_mode_tx_uart.recv_end == SET)
    {
      uart_tx_total_length += tp_mode_tx_uart.count;
      tp_mode_tx_uart.recv_end = RESET;
      if(tp_mode_tx_uart.tail > (USART_RECV_LEN - 1))
      {
        tp_mode_tx_uart.tail = 0;
      } 
      for(i = 0; i < tp_mode_tx_uart.count; i++)
      {
        while(usart_flag_get(USART3, USART_TDBE_FLAG) !=SET);
        usart_data_transmit(USART3, tp_mode_tx_uart.buf[i]);
      }
      memset(tp_mode_tx_uart.buf, 0, USART_RECV_LEN);
      tp_mode_tx_uart.count = 0;
      tp_mode_tx_uart.head = 0;
      tp_mode_tx_uart.tail = 0;
    }
  }
}
